@@ -16,12 +16,12 @@ use strict;
 use warnings;
 no warnings 'once';
 
-our $VERSION = '0.007012';
+our $VERSION = '0.008000';
 $VERSION = eval $VERSION;
 
 use Carp ();
 use Symbol ();
-use Devel::Confess::_Util qw(blessed refaddr weaken longmess _str_val _in_END);
+use Devel::Confess::_Util qw(blessed refaddr weaken longmess _str_val _in_END _can_stringify);
 use Config ();
 BEGIN {
   *_can = \&UNIVERSAL::can;
@@ -32,11 +32,12 @@ BEGIN {
     = ($] > 5.008009 && $] <= 5.010000) ? sub () { 1 } : sub () { 0 };
   *_BROKEN_SIG_DELETE
     = ($] < 5.008008) ? sub () { 1 } : sub () { 0 };
-  my $debugging = defined &Config::non_bincompat_options
-    ? (Config::non_bincompat_options() =~ /\bDEBUGGING\b/)
-    : ($Config::Config{ccflags} =~ /-DDEBUGGING\b/);
   *_DEBUGGING
-    = $debugging ? sub () { 1 } : sub () { 0 };
+    = (
+      defined &Config::non_bincompat_options
+        ? (grep $_ eq 'DEBUGGING', Config::non_bincompat_options())
+        : ($Config::Config{ccflags} =~ /-DDEBUGGING\b/)
+    ) ? sub () { 1 } : sub () { 0 };
 }
 
 $Carp::Internal{+__PACKAGE__}++;
@@ -155,6 +156,7 @@ sub _find_sig {
   my $sig = $_[0];
   return undef
     if !defined $sig;
+  local $@;
   return $sig
     if ref $sig && eval { \&{$sig} };
   return undef
@@ -168,11 +170,12 @@ sub _find_sig {
 sub _warn {
   local $SIG{__WARN__};
   my @convert = _convert(@_);
-  if (my $warn = _find_sig($OLD_SIG{__WARN__})) {
-    $warn->(join('', @convert));
+  if (my $sig = _find_sig($OLD_SIG{__WARN__})) {
+    $sig->(ref $convert[0] ? $convert[0] : join('', @convert));
   }
   else {
-    _colorize(\@convert, 33) if $OPTIONS{color};
+    @convert = _ex_as_strings(@convert);
+    @convert = _colorize(33, @convert) if $OPTIONS{color};
     warn @convert;
   }
 }
@@ -180,32 +183,29 @@ sub _die {
   local $SIG{__DIE__};
   my @convert = _convert(@_);
   if (my $sig = _find_sig($OLD_SIG{__DIE__})) {
-    $sig->(join('', @convert));
+    $sig->(ref $convert[0] ? $convert[0] : join('', @convert));
   }
-  _colorize(\@convert, 31) if $OPTIONS{color} && !$^S;
+  @convert = _ex_as_strings(@convert) if _can_stringify;
+  @convert = _colorize(31, @convert) if $OPTIONS{color} && _can_stringify;
   if (_DEBUGGING && _in_END) {
     local $SIG{__WARN__};
     warn @convert;
     $! ||= 1;
     return;
   }
-  die @convert;
+  die @convert unless ref $convert[0];
 }
 
 sub _colorize {
-  my ($convert, $color) = @_;
+  my ($color, @convert) = @_;
   if ($ENV{DEVEL_CONFESS_FORCE_COLOR} || -t *STDERR) {
-    if (blessed $convert->[0]) {
-      if ($convert->[0]->isa('Devel::Confess::_Attached')) {
-        splice @$convert, 0, 1, _ex_as_strings($convert->[0]);
-      }
-      else {
-        $convert->[0] =~ s/(.*)/\e[${color}m$1\e[m/;
-        return;
-      }
+    if (@convert == 1) {
+      $convert[0] = s/(.*)//;
+      unshift @convert, $1;
     }
-    $convert->[0] = "\e[${color}m$convert->[0]\e[m";
+    $convert[0] = "\e[${color}m$convert[0]\e[m";
   }
+  return @convert;
 }
 
 sub _ref_formatter {
@@ -319,7 +319,7 @@ sub _convert {
     }
 
     bless $ex, $newclass;
-    $ex;
+    return $ex;
   }
   elsif (ref($ex = $_[0])) {
     my $id = refaddr($ex);
@@ -330,7 +330,7 @@ sub _convert {
     $PACKAGES{$id} = undef;
     $MESSAGES{$id} ||= $message;
 
-    return ($^S ? @_ : ( @_, $message ));
+    return $ex;
   }
   elsif ((caller(1))[0] eq 'Carp') {
     my $out = join('', @_);
@@ -359,13 +359,29 @@ sub _convert {
 
 sub _ex_as_strings {
   my $ex = $_[0];
+  return @_
+    unless ref $ex;
   my $id = refaddr($ex);
   my $class = $PACKAGES{$id};
   my $message = $MESSAGES{$id};
-  my $newclass = ref $ex;
-  bless $ex, $class;
-  my $out = "$ex";
-  bless $ex, $newclass;
+  my $out;
+  if (blessed $ex) {
+    my $newclass = ref $ex;
+    bless $ex, $class if $class;
+    if ($OPTIONS{dump} && !overload::OverloadedStringify($ex)) {
+      $out = _ref_formatter($ex);
+    }
+    else {
+      $out = "$ex";
+    }
+    bless $ex, $newclass if $class;
+  }
+  elsif ($OPTIONS{dump}) {
+    $out = _ref_formatter($ex);
+  }
+  else {
+    $out = "$ex";
+  }
   return ($out, $message);
 }
 
@@ -445,11 +461,11 @@ Devel::Confess - Include stack traces on all warnings and errors
 Use on the command line:
 
   # Make every warning and error include a full stack trace
-  perl -MDevel::Confess script.pl
-
-  # equivalent short form
   perl -d:Confess script.pl
 
+  # Also usable as a module
+  perl -MDevel::Confess script.pl
+
   # display warnings in yellow and errors in red
   perl -d:Confess=color script.pl
 
@@ -478,13 +494,13 @@ of errors.  L<Carp>'s C<carp> and C<confess> functions will also be made to
 include stack traces.
 
   # it works for explicit die's and warn's
-  $ perl -MDevel::Confess -e 'sub f { die "arghh" }; sub g { f }; g'
+  $ perl -d:Confess -e 'sub f { die "arghh" }; sub g { f }; g'
   arghh at -e line 1.
           main::f() called at -e line 1
           main::g() called at -e line 1
 
   # it works for interpreter-thrown failures
-  $ perl -MDevel::Confess -w -e 'sub f { $a = shift; @a = @$a };' \
+  $ perl -d:Confess -w -e 'sub f { $a = shift; @a = @$a };' \
                                         -e 'sub g { f(undef) }; g'
   Use of uninitialized value $a in array dereference at -e line 1.
           main::f(undef) called at -e line 2
